###
### Preprocess.mk
###
### Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of Janos.
### 
### Janos is free software; you can redistribute it and/or modify it
### under the terms of the GNU General Public License, version 2, as
### published by the Free Software Foundation.
### 
### Janos is distributed in the hope that it will be useful, but
### WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details, in the file
### COPYING.  If that file is not present, write to the Free
### Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
### 02111-1307, USA
### 
### The University of Utah and the Flux Group requests users to
### return to flux-dist@cs.utah.edu any improvements that they make.
###

###
### Preprocessor stuff.  
###
### This file must define the makefile variables PREPROCESS and DOC_MAGIC_SUBST.
### They will be used like this: 
###     '$(PREPROCESS) [infile] >> [outfile]'
### Also, DefinePackage.mk depends on CPP_GLOBAL_MACROS as a dependency.
###
### This file should define no targets (other than debug::)
###
### The following variables can be expected to be defined:
###  BUILD, LIBSTYLE, PACKAGE_PREFIX, PACKAGE_PREFIX_DIR, DEST_DIR, SRC_DIR
###
### XXX Watch out for name clashes, only one namespace in make.
###

# The CPP definition used throughout the Java code
CPP_GLOBAL_MACROS:=$(SRC_DIR)/Defines.cpp

# The actual cpp "executable". 
#   -E: make gcc just run the preprocessor
#   -x c: forces gcc to consider the input "C" code, despite the .Java suffix
#   -P:no explicit line/file info  (though this messes up the implicit line numbers!).
#   -Wcomment: warn about bad comments
#   -nostdinc: use none of the standard #include directories
#   -undef: no pre-defined macros 
#   -C: leave comments in preprocessed source
ifndef PREPROCESSOR
PREPROCESSOR:=gcc -E -x c -P -Wcomment -nostdinc -undef -C
endif

# Use different CPP flags based on the build
CPP_FLAGS_debug:=-DDEBUG -UEXPOSED -UNDEBUG -DTEST
CPP_FLAGS_optimized:=-UEXPOSED -UDEBUG -DNDEBUG
CPP_FLAGS_exposed:=-UNDEBUG -DDEBUG -DEXPOSED -DTEST

CPP_FLAGS:=$(CPP_FLAGS_$(BUILD)) -I. -imacros $(CPP_GLOBAL_MACROS)
#   -I.: search the current directory for -imacros file
#   -imacros <file>: process <file> and keep macro/define info

### The whole point of this file:
PREPROCESS:=$(PREPROCESSOR) $(CPP_FLAGS)


### Sanity check rule in case something happens to the macros file
$(CPP_GLOBAL_MACROS): 
	@echo "The global macro file $(CPP_GLOBAL_MACROS) doesn't exist.  Something is broken."
	@$(FAILURE)

###
### The ubiquitous debug target
###

debug:: debug_Preprocess.mk

debug_Preprocess.mk::
	@echo " "
	@echo "**** Preprocess.mk settings:"
	@echo " "
	@echo "    CPP                 = $(CPP)"
	@echo "    CPP_GLOBAL_MACROS   = $(CPP_GLOBAL_MACROS)"
	@echo "    CPP_FLAGS           = $(CPP_FLAGS)"
	@echo "    PREPROCESS          = $(PREPROCESS)"

.PHONY: debug_Preprocess.mk
