/* 
 * PongCapsule.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package JANOS_ANTSR_PONG_APP_PKG;

import JANOS_ANTSR_CORE_PKG.Protocol;
import JANOS_ANTSR_CORE_PKG.DataCapsule;
import JANOS_ANTSR_CORE_PKG.Xdr;
import JANOS_ANTSR_CORE_PKG.Node;
import JANOS_ANTSR_CORE_PKG.ByteArray;

/**
 * Pong capsule.
 *
 * @author David Wetherall
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a> 
 */
public class PongCapsule
	extends DataCapsule
{
	final private static byte[] MID = findMID(JANOS_ANTSR_PONG_APP_PKG_S + ".PongCapsule");
	final private static byte[] PID = findPID(JANOS_ANTSR_PONG_APP_PKG_S + ".PongCapsule");

	protected byte[] mid()
	{
		return MID;
	}
	
	protected byte[] pid()
	{
		return PID;
	}
	
	public boolean pong = false;

	/**
	 * Sequence number for the pong capsule.  
	 */
	public int seqId;
	
	public int length()
	{
		return super.length() + Xdr.BOOLEAN + Xdr.INT;
	}
  
	public Xdr encode()
	{
		Xdr xdr = super.encode();
		xdr.PUT(pong);
		xdr.PUT(seqId);
		
		return xdr;
	}
	
	public Xdr decode()
	{
		Xdr xdr = super.decode();
		pong = xdr.BOOLEAN();
		seqId = xdr.INT();
		
		return xdr;
	}
	
	public boolean evaluate(Node n)
	{
		PongHelper ph = new PongHelper();

		ph.setDummy1(0);
		ph.setDummy2(1);
		if( ph.getDummy1() != 0 )
			throw new InternalError();
		if( ph.getDummy2() != 1 )
			throw new InternalError();
		if (n.getAddress() == getDst())
		{
			pong = true;      
		}
		else if (pong!=true)
		{
			return n.routeForNode(this, getDst());
		} 

		if (n.getAddress() == getSrc()) 
		{
			return n.deliverToApp(this, dpt);
		}
		else if (pong)
		{
			return n.routeForNode(this, getSrc());
		}
		
		return false; 
	}
	
	public PongCapsule()
	{
	}
	
	public PongCapsule(short sa, short da, int na, ByteArray d)
	{
		super(sa, da, na, d);
	}
}

