/*
 * ANTSRDemultiplexKey.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_CORE_PKG;

IMPORT_DEBUG
IMPORT_TRACE

/**
 * Match ANTSR packets for a specific ANTSR protocol.
 */
PACKAGE class ANTSRDemultiplexKey
	extends ANEPDemultiplexKey
{
	private final byte[] midBytes;

	PACKAGE ANTSRDemultiplexKey(TypeID mid)
	{
		this(mid.unwrap());
	}

	PACKAGE ANTSRDemultiplexKey(byte[] midBytes)
	{
		super(midBytes.length, Capsule.ANTSRTYPE);
		this.midBytes = midBytes;
	}

	public int matches(byte[] buf, int packetOffset, int packetLength)
	{
		int anepMatch;

		anepMatch = super.matches(buf, packetOffset, packetLength);

		if (anepMatch == 0)
			return 0;
		assert(anepMatch == 4);

		// XXX Malformed packet
		if (packetLength < 8)
			return 0;
		
		// Get the option header length (stored as 32-bit word count)
		int headerLen = (((buf[packetOffset + 4] << 8)
				  | (buf[packetOffset + 5]))) * 4;
		if (headerLen < 8) // minimum allowed value.
		{
			TRACE(Trace.ANTSR_DEMULTIPLEX, "ANTSKey Malformed packet: headerlen="+headerLen+ " < 8.");
			return 0;
		}

		// XXX Malformed packet
		if (headerLen > packetLength)
		{
			TRACE(Trace.ANTSR_DEMULTIPLEX, "ANTSRKey Malformed packet: headerlen="+headerLen+ "; packetLen=" +packetLength+ ".");
			return 0;
		}
		
		return compareSlice(buf, packetOffset, packetLength,
				    headerLen + Xdr.INT, midBytes);
	}

	/**
	 * Really only useful for subclasses....
	 */
	public int whereDoesTheCapsuleDecodingActuallyStart(int payloadStart)
	{
		return payloadStart;
	}

	public String toString()
	{
		StringBuffer buf = new StringBuffer(100);

		buf.append(super.toString());
		buf.append("+ANTSRKey[");
		assert(midBytes != null);
		assert(midBytes.length == 16);
		for (int i = 0; i < midBytes.length; i++)
			buf.append(Integer.toHexString((midBytes[i])&0xFF));
		buf.append("]");

		return buf.toString();
	}

	/**
	 * Tests if the super class says they're equal and then compares
	 * they're MID's
	 */
	public boolean equals(Object obj)
	{
		boolean retval = false;

		if( (obj instanceof ANTSRDemultiplexKey) &&
		    (this.getClass().equals(obj.getClass())) )
		{
			ANTSRDemultiplexKey adk = (ANTSRDemultiplexKey)obj;

			if( super.equals(obj) )
			{
				int lpc;

				retval = true;
				for( lpc = 0;
				     (lpc < this.midBytes.length) && retval;
				     lpc++ )
				{
					retval = (this.midBytes[lpc] ==
						  adk.midBytes[lpc]);
				}
			}
		}
		return retval;
	}
}
