/* 
 * PendingRequest.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_ANTSR_CORE_PKG;

import JANOS_IO_PKG.Log;

IMPORT_TRACE

/**
 * Used to track request capsules that are still need a response.
 */
PACKAGE class PendingRequest
	extends NodeCache.EventHandler
{
	/**
	 * The request capsule
	 */
	private Capsule cap;

	/**
	 * Amount of time to wait before retrying
	 */
	private static final int defaultRetryTime = 2;

	/**
	 * The number of times to retry
	 */
	private static final int defaultRetryCount = 5;

	/**
	 * The ttl to reprime the capsule with, this increases with each
	 * retries to make sure we can get the thing out.
	 */
	private int ttl = 1;
	
	/**
	 * The number of times left to retry sending the capsule.
	 */
	private int retries = defaultRetryCount;

	/**
	 * Construct a pending request for this capsule
	 */
	public PendingRequest(Capsule cap)
	{
		this.cap = cap;
	}

	/**
	 * Return the request capsule
	 */
	public Capsule getCapsule()
	{
		return this.cap;
	}

	/**
	 * Event handler called by NodeCache when we're being collected.
	 * This just checks if we have any retries left and sends the capsule
	 * if we do.
	 */
	public int collected()
	{
		int retval;
		
		if( this.retries == 0 )
		{
			TRACE(Trace.ANTSR_OUTGOING, "Request capsule wasn't received " +this.cap);
			retval = 0;
			Node.currentNode().shutdown();
		}
		else
		{
			Node node = Node.currentNode();

			TRACE(Trace.ANTSR_OUTGOING, "Resending capsule " +this.cap);
			this.cap.prime(node.getAddress(), this.ttl);
			node.routeForNode(this.cap, this.cap.getDst());
			retval = defaultRetryTime;
			this.retries--;
			this.ttl++;
		}
		return retval;
	}
}
