/*
 * RouteTable.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_CORE_PKG;

import JANOS_NODEOS_PKG.AddressFormatException;
import JANOS_NODEOS_PKG.NoSuchNodeAddress;

import JANOS_ROUTE_PKG.RouteEntry;

import java.io.IOException; // XXX

import java.util.Enumeration; // XXX


/**
 * This is the interface the ants.ants package expects 
 * a route table to satisfy.
 * <p>
 * This class only exists to avoid circular dependencies
 * between the ants.ants package and any dynamic routing
 * table implementation.  XXX We may be going to far.
 * <p>
 * XXX notice that we're still dependent upon the
 * Janos route package's RouteEntry type.  Ah, well.
 * <p>
 * XXX The name is a bit confusing, especially for code
 * that can see this class and janos/route/RouteTable.
 */
public interface RouteTable
{
	/**
	 * Configure the table.
	 * <p>
	 * XXX the exception throw is a bit overkill...
	 */
	void configure(int nodeAddr,
			  boolean doUpdates,
			  String configFile)
		throws IOException, AddressFormatException, NoSuchNodeAddress;
		
	/**
	 * Must return an enumeration of RouteEntry objects.
	 */
	Enumeration getNeighbors();

	int neighborCount();

	RouteEntry getRoute(int addr);
}


