/*
 * DynamicRouteEntry.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_ROUTE_PKG;

import JANOS_NODEOS_PKG.NoSuchNodeAddress;
import JANOS_NODEOS_PKG.AddressFormatException;
import JANOS_NODEOS_PKG.Address;

import JANOS_ROUTE_PKG.RouteEntry;

IMPORT_DEBUG
IMPORT_TRACE

/**
 * A DynamicRouteEntry is a RouteEntry with weight.
 * <p>
 * Unlike a base RouteEntry, which is immutable, a 
 * DynamicRouteEntry's cost, expires and change fields
 * are mutable.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 * @author David Wetherall
 */
PACKAGE class DynamicRouteEntry
	extends RouteEntry
{
	// Cost stuff used by the RouteTable update code.
	private int cost;
	private long expires;
	PACKAGE boolean change;
	
	public DynamicRouteEntry(int dest, int next, String physAddr)
		throws NumberFormatException, AddressFormatException, NoSuchNodeAddress
	{
		super(dest, next, physAddr);
		change = false;
		cost = -1; // XXX
		expires = -1; // XXX
	}
	
	public DynamicRouteEntry(DynamicRouteEntry other)
	{
		super(other);
		change = other.change;
		cost = other.cost;
		expires = other.expires;
	}

	public int getCost()
	{
		assert(cost > 0); // it had best have been set before being querried.
		return cost;
	}

	public void setCost(int cost)
	{
		this.cost = cost;
	}

	public long getExpiration()
	{
		assert(expires > 0); // it had best have been set before being querried.
		return expires;
	}

	public void setExpiration(long expires)
	{
		this.expires = expires;
	}

	public boolean changed()
	{
		return change;
	}

	public void markChanged()
	{
	}

	public void clearChanged()
	{
	}

	public String toString() {
		return "DynamicRouteEntry["
			+ "d=" +Address.toString(getLogicalDest()) + ";"
			+ "n=" +Address.toString(getLogicalNext()) + ";"
			+ "a=" +getPhysicalNext() + ";"
			+ "$=" +getCost()
			+ "]";
	}
}
