/* 
 * DynamicRouteEvent.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 */
package JANOS_ANTSR_ROUTE_PKG;

IMPORT_DEBUG

/**
 * A timer event affecting the DynamicRouteTable entries.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 * @author David Wetherall
 */
PACKAGE class DynamicRouteEvent 
{
	// different types of route events:

	/**
	 * An EMPTY event.  DynamicRouteEvents are never
	 * initialized with this value.  They just
	 * decay to it after they expire.
	 */
	PACKAGE final static int EMPTY    = 0;

	/**
	 * The PERODIC event.  These trigger periodically (duh)
	 * and send a full update to all neighbor gateways.
	 */
	PACKAGE final static int PERIODIC = 1<<0;

	/**
	 * The TRIGGER event.  Trigger events are scheduled for
	 * a short time in the future to propagate recent
	 * changes.  
	 */
	PACKAGE final static int TRIGGER  = 1<<1;

	/**
	 * The GC event is triggered on a route when it is
	 * time for the route to be retired.  Routes are
	 * first moved into a gc pool where they can be
	 * ressurected.  Otherwise they will EXPIRE.
	 */
	PACKAGE final static int GC       = 1<<2;

	/**
	 * The EXPIRE event.  This event is triggered on a route
	 * after its been placed in the GC pool and if it
	 * doesn't get resurrected.  EXPIRED routes are deleted.
	 */
	PACKAGE final static int EXPIRE   = 1<<3;
	
	/**
	 * Timestamp of when this event is scheduled to
	 * be invoked.
	 */
	PACKAGE final long when;

	/**
	 * A set of flags indicating what sort of events this
	 * object implies (Periodic, trigger, expire, or GC).
	 * <p>
	 * This is mutable.  Events may be added if they occur
	 * at the same time as a different event.
	 */
	PACKAGE int what;

	/**
	 * Let me know if you figure out what this does.
	 */
	PACKAGE int expireCount;

	/**
	 * Let me know if you figure out what this does.
	 */
	PACKAGE int gcCount;
	
	PACKAGE DynamicRouteEvent(long when, int what) 
	{
		assert(what != EMPTY);
		assert(when > 0);

		this.when = when;
		this.what = what;
		this.expireCount = 0;
		this.gcCount = 0;
	}
}

