/*
 * Buffer.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * The Buffer that a BufferHandle points to.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 */
PACKAGE class Buffer
{
	private int refCount;
	PACKAGE byte[] data;
	
#ifdef TEST
	private boolean recycled = false;
#endif
	
	PACKAGE Buffer(byte[] data)
	{
		this.data = data;
		refCount = 1;
	}

	/**
	 * Return true if the flow can afford the
	 * buffer.
	 */
	PACKAGE boolean charge(FlowState state)
	{
		// XXX already charged the flow...
		return true;
	}
		
	PACKAGE final synchronized void acquire()
	{
		refCount++;
	}

	PACKAGE final synchronized void release()
	{
		if(refCount == 0) throw new IllegalStateException();
		refCount--;
		if(refCount == 0) recycle();
	}

	// This can be overridden if necessary.
	PACKAGE void recycle()
	{
#ifdef TEST
		recycled = true;
#endif
	}

#ifdef DEBUG
	PACKAGE final int getRefCount() {return refCount;}
#endif // DEBUG
	
	public String toString()
	{
		return "Buffer[refCount="
			+ refCount
			+ "; data["
			+ data.length
			+ "]]";
	}
}
