/*
 * CutThroughChannel.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

/**
 * Java wrapper for a CutThroughChannel.
 * <p>
 * Implemented on top of UDP for the moment.
 * 
 * @see InChannel
 * @see OutChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 */
public final class CutThroughChannel
{
	/**
	 * The OutChan this CutThrough forwards to.
	 */
	private final OutChannel outTo;

	/**
	 * The InChan this CutThrough receives from.
	 */
	private final InChannel inFrom;

	public CutThroughChannel(Flow flow,
				 DemultiplexKey classifier,
				 ThreadPool threadPool,
				 Address destAddr)
		throws IOException
	{
		// XXX this isn't sufficient protection for the rest of the kernel..
		if((flow == null)
		   || (threadPool == null)
		   || (classifier == null))
			throw new NullPointerException();

		outTo = new OutChannel(flow, destAddr);
		// above might throw IOException

		DeliverFunc f = new DeliverFunc() {
			public void receive(BufferHandle packet, int payloadStart)
				throws IOException
				{
					outTo.send(packet);
				}
		};

		inFrom = new InChannel(flow,
				       classifier,
				       threadPool,
				       f);

		flow.addChannel(this);
	}

	PACKAGE void destroy()
	{
		TRACE(Trace.NODEOS_DESTROY, "Destroying " + this);
	}
}


