/*
 * FlowState.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Per-flow data.
 * <p>
 * XXX why is this separate from the Flow?
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 * @see Flow
 */
PACKAGE final class FlowState
{
	/**
	 * XXX: need to examine the global locking structure for
	 * deadlock, etc
	 * <p>
	 * XXX: Probably should move to locks on each element of the
	 * state and not a global state lock.
	 */
	PACKAGE final Object lock = new Object();

	// Used by UdpBuffer
	PACKAGE int numBuffers;
	PACKAGE int freeBuffers;

	// Used by PacketWorker
	PACKAGE PacketWorker packetWorkerFreeList;

	PACKAGE FlowState(MemSpec memSpec)
	{
		numBuffers = memSpec.numBuffers;
		freeBuffers = numBuffers;
		UdpBuffer.addBuffers(numBuffers);
	}

	PACKAGE void init(Flow flow)
	{
		PacketWorker.init(flow);
	}

	public String toString()
	{
		return "FlowState[numBuffers="
			+ numBuffers
			+ "; freeBuffers="
			+ freeBuffers
			+ "]";
	}
}
