/*
 * InChannelHandle.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;


/**
 * A cross-process handle on an InChannel.
 * Packets can be inserted into the InChannel via
 * an InChannelHandle.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 * @see InChannel
 */
public final class InChannelHandle
{
	private InChannel inChan;

	public InChannelHandle()
	{
		setChannel(null);
	}

	public InChannelHandle(InChannel inChan)
	{
		setChannel(inChan);
	}

	public void setChannel(InChannel inChan)
	{
		this.inChan = inChan;
	}

	public void copy(InChannelHandle otherChan)
	{
		setChannel(otherChan.inChan);
	}

	/**
	 * @allocation an OffsetLength object for hokey reasons.
	 */
	public boolean push(BufferHandle pkt, int payloadStart)
	{
		TRACE(Trace.NODEOS_OUTGOING, "InChannelHandle.push: " +Debug.dbgPacketSummary(pkt, 128));
		OffsetLength ol = new OffsetLength();  // ACK!
		Buffer buf;
		assert(inChan); // XXX
		boolean retval;
		
		// Grab the Buffer, offset and length
		if( pkt.getNext() == pkt && ol.offset == 0 )
		{
			buf = pkt.getBuffer(ol);
		}
		else
		{
			/* The buffer handle is a list so we aggregate them */
			synchronized(inChan.flow.state.lock)
			{
				byte data[] = new byte[BufferHandle.
						      aggregateLength(pkt)];

				BufferHandle.aggregate(data, pkt);
				buf = new Buffer(data);
			}
		}

		/*
		 * Push into a new BufferHandle in target process's
		 * context.
		 */
		retval = inChan.scheduleDelivery(buf, ol.offset, ol.length,
						 payloadStart);
		buf.release();
		return retval;
	}

	public String toString()
	{
		return "InChannelHandle[inChan=" +inChan+ "]";
	}
}

