/*
 * Main.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Boot the Janos Java NodeOS.
 * <p>
 * Strives for swanhood.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 */
public class Main
{
	PACKAGE final static int ROOT_FLOW_BUFFERS = 32;

	/**
	 * All Janos active routers start with this function. 
	 * We load up native libraries, define the set of
	 * "system" packages, and fire up the root flow.
	 * The arguments to main() should be a class to instantiate
	 * in the root flow and whatever arguments it wants.
	 *
	 * @see FlowClassLoader for info system packages.
	 */
	public static void main(String[] args)
	{
		String initClass;
		String initPackage;
		String[] initArgs;
		int pkgend;

		// Load native libraries, etc.
		loadNativeLayer();

		if (args.length == 0)
		{
			Node.println("Must supply a class to load into root flow.");
			Node.shutdown();
		}

		initClass = args[0];

		pkgend = initClass.lastIndexOf('.');
		initPackage = null;
		if (pkgend != -1)
			initPackage = initClass.substring(0, pkgend);

		initArgs = null;
		if (args.length > 1) 
		{
			initArgs = new String[args.length - 1];
			System.arraycopy(args, 1, initArgs, 0, initArgs.length);
		}
		

		/**
		 * Define the set of "system" packages that are loaded
		 * into the system flow.  This set is basically fixed,
		 * except for the implicit package passed on the command
		 * line...
		 */
		FlowClassLoader.addSystemPackage("java.lang");
		FlowClassLoader.addSystemPackage("java.io");
		FlowClassLoader.addSystemPackage("java.util");
		FlowClassLoader.addSystemPackage(JANOS_NODEOS_PKG_S);
		FlowClassLoader.addSystemPackage(JANOS_UTIL_PKG_S);
		FlowClassLoader.addSystemPackage(JANOS_IO_PKG_S);
		FlowClassLoader.addSystemPackage(JANOS_ROUTE_PKG_S);
		if (initPackage != null)
			FlowClassLoader.addSystemPackage(initPackage);

		Node.println("Janos Java NodeOS v1.0");

		// create the root flow
		Flow rootFlow = new Flow(
			null,
			new Credentials("root cred"),
			new ThreadPoolSpec(
				null,
				ThreadScheduler.basicScheduler()),
			new MemSpec(ROOT_FLOW_BUFFERS),
			new FlowStartSpec(
				new ClassSpec[0],
				initClass,
				initArgs,
				/*CommSpaceHandle*/ null));
		DEBUGONLY(rootFlow.setName("Root Flow"));
	}

	private static void loadNativeLayer()
	{
		// java.lang.System.loadLibrary("JanosNodeOS");
	}
}

