/*
 * PacketWorker.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Used by Channel.
 * <p>
 * XXX could this be a subclass of WorkItem?
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 */
PACKAGE class PacketWorker 
	implements Runnable
{
	PACKAGE final Flow flow;
	PACKAGE PacketWorker nextFree;
	PACKAGE final WorkItem workItem;
	PACKAGE final BufferHandle bufferHandle;
	PACKAGE int payloadStart;
	PACKAGE DeliverFunc deliverFunc;

	PACKAGE static void init(Flow flow)
	{
		for(int i = 0; i < flow.state.numBuffers; i++)
		{
			PacketWorker p = new PacketWorker(flow);
			p.nextFree = flow.state.packetWorkerFreeList;
			flow.state.packetWorkerFreeList = p;
		}
	}

	/* returns null if no workers are available */
	PACKAGE static PacketWorker getFree(Flow flow)
	{
		FlowState state = flow.state;
		synchronized(state.lock)
		{
			PacketWorker p = state.packetWorkerFreeList;
			if (p != null)
			{
				state.packetWorkerFreeList = p.nextFree;
				p.nextFree = null;
			}
			return p;
		}
	}

	PACKAGE PacketWorker(Flow flow)
	{
		this.flow = flow;
		workItem = new WorkItem(this);
		bufferHandle = new BufferHandle(flow);
	}

	public void run()
	{
		try
		{
			Thread.currentThread().setName("PacketWorker for " + this.deliverFunc.getClass().getName());
			BENCHONLY(InChannel.threadSchedTimer.end());
			deliverFunc.receive(bufferHandle, payloadStart);
		}
		catch(Throwable t)
		{
			java.io.StringWriter s = new java.io.StringWriter();
			t.printStackTrace(new java.io.PrintWriter(s));
			TRACE(Trace.NODEOS_OUTGOING, "uncaught exception in deliverFunc:\n" +s);
		}
		/*
		 * This is kinda icky, we have to syncrhonize here since we
		 * could free the buffer for use before the corresponding
		 * packet worker is placed back on the list
		 */
		synchronized(flow.state.lock)
		{
			bufferHandle.reset();
			
			addToFreeList(flow.state, this);
		}
		Thread.currentThread().setName("Empty PacketWorker");
	}

	private static void addToFreeList(FlowState state, PacketWorker p)
	{
		synchronized(state.lock)
		{
			assert(p.nextFree == null);
			p.nextFree = state.packetWorkerFreeList;
			state.packetWorkerFreeList = p;
		}
	}
}


