/*
 * ProtocolRegister.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.util.Enumeration; // XXX
import java.util.Hashtable; // XXX

/**
 * Maintain a map of all the Flows in the system.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 */
public class ProtocolRegister
{
	private static final Hashtable protocols = new Hashtable();

	/**
	 * A simple class used to identify who owns a protocol
	 */
	public static class Owner
	{
		public String id;
		public String data;

		public Owner(String id, String data)
		{
			this.id = id;
			this.data = data;
		}
	}

	private static class ProtocolInfo
		extends InfoHook
	{
		public ProtocolInfo()
		{
			super("Protocols");
		}

		public String[] getInfo()
		{
			String retval[];
			
			synchronized(protocols)
			{
				retval = new String[protocols.size()];
				Enumeration enum = protocols.elements();
				int i;
				
				for( i = 0; i < protocols.size(); i++ )
				{
					Owner o = (Owner)enum.nextElement();

					retval[i] = "id="
						+ o.id
						+ "; data="
						+ o.data;
				}
			}
			return retval;
		}
	}

	static {
		Status.addInfoHook(new ProtocolInfo());
	}
	
	/**
	 * Register a protocol, if the protocol has already been registered
	 * then the current owner is returned, otherwise null is returned
	 * and caller is now the owner.
	 *
	 * @param id An object used to identify a protocol
	 * @param owner The entity who wants to be the owner of the protocol
	 * @return The current owner or null, if none
	 */
	public static Owner register(Object id, Owner owner)
	{
		Owner retval;

		synchronized(protocols)
		{
			retval = (Owner)ProtocolRegister.protocols.get(id);
			if( retval == null )
				ProtocolRegister.protocols.put(id, owner);
		}
		return retval;
	}
}
