/*
 * Status.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.util.Hashtable; // XXX
import java.util.Enumeration; // XXX

/**
 * Obtain status information on the state of a node.
 * <p>
 * This is the public interface to querying stats on the
 * node.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu>Janos Ministry of Development</a>
 */
public class Status
{
	private static final Status statusObj = new Status();

	/**
	 * A `root' InfoHook that is used to track the rest of them
	 */
	private static final InfoHook rootHook = new InfoHook("root");

	/**
	 * Private constructor.  Only one status object is ever
	 * created, the one referenced in statusObj.
	 */
	private Status()
	{
	}

	private static boolean gaveStat = false;

	/**
	 * Return the single global status object exactly once.
	 * First come, first served.  The caller can decide what
	 * sort of access controls to implement.
	 */
	public static Status getStatusObject()
	{
		if (gaveStat)
			return null;
		gaveStat = true;
		return statusObj;
	}

	/**
	 * Add an InfoHook that can be inspected by the Status user
	 */
	public static void addInfoHook(InfoHook ih)
	{
		rootHook.addChild(ih);
	}

	/**
	 * Get the information string corresponding to the given module name
	 *
	 * @param name The name of the InfoHook to lookup
	 */
	public String[] getInfo(String name)
	{
		String retval[] = null;
		InfoHook child;

		child = rootHook.findChild(name);
		if( child != null )
		{
			retval = child.getInfo();
		}
		return retval;
	}

	/**
	 * Get the list of InfoHook's available
	 *
	 * @return An array of strings containing the names of the info hooks
	 */
	public String[] getInfoList()
	{
		return rootHook.getChildList();
	}
	
}
