/*
 * AddressTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_TESTS_NODEOS_PKG;

import java.net.InetAddress;

import JANOS_TESTS_PKG.*;

import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFormatException;

public class AddressTester extends Tester
{
	public void testUnit() throws TestFailedException
	{
		Address addr, addrCopy;
		InetAddress iaddr;
		int lpc;

		try
		{
			iaddr = InetAddress.getLocalHost();
		}
		catch(java.net.UnknownHostException e)
		{
			return;
		}
		
		this.enterSection(ENV_NONE, "Valid fromString()/toString()'s");
		try
		{
			String addrStrings[] =
			{
				"0.0.0.0",
				"0.1.2.4",
				"8.16.32.64",
				"255.255.255.255"
			};
			int addrInts[] =
			{
				0x00000000,
				0x00010204,
				0x08102040,
				0xFFFFFFFF
			};

			for( lpc = 0; lpc < addrStrings.length; lpc++ )
			{
				this.test(Address.
					  fromString(addrStrings[lpc])
					  == addrInts[lpc]);
				this.test(Address.toString(addrInts[lpc]).
					  equals(addrStrings[lpc]));
			}
		}
		catch(AddressFormatException e)
		{
			throw new TestFailedException(this);
		}

		this.enterSection(ENV_NONE, "Invalid fromString()'s");
		String badAddrs[] =
		{
			"",
			"256.256.256.256",
			"0",
			"0.0",
			"0.0.0",
			"0.0.0. ",
			"0.0.0.a",
			"This.really.shouldn't.work",
			"Nor this",
			"0 . 0 . 0 . 0",
			"-1.-1.-1.-1"
		};

		for( lpc = 0; lpc < badAddrs.length; lpc++ )
		{
			try
			{
				Address.fromString(badAddrs[lpc]);
				throw new TestFailedException(this);
			}
			catch(AddressFormatException e)
			{
				; /* This is supposed to happen */
			}
		}

		this.enterSection(ENV_EXPOSED, "getAddressFactory()");
#ifdef EXPOSED
		Address.gaveFactory = false;
		
		if( Address.getAddressFactory() == null )
			throw new TestFailedException(this);
		if( Address.getAddressFactory() != null )
			throw new TestFailedException(this);

		/* Restore it so we don't screw anything else up */
		Address.gaveFactory = false;
#endif
		
		this.enterSection(ENV_EXPOSED, "Constructors/Accessors");
#ifdef EXPOSED
		/* Regular constructor */
		addr = new Address(0x00010204, iaddr, 1024);
		addrCopy = new Address(addr);
		this.test(addr.getANAddress() == 0x00010204);
		this.test(addr.getANAddress() == addrCopy.getANAddress());
		this.test(addr.iNetAddr() == iaddr);
		this.test(addr.iNetPort() == 1024);
		this.test(addr.iNetAddr() == addrCopy.iNetAddr());
		this.test(addr.iNetPort() == addrCopy.iNetPort());
#endif
	}
}
