/*
 * SimpleDemultiplexKey.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_UTIL_PKG;

import JANOS_NODEOS_PKG.DemultiplexKey;

public class SimpleDemultiplexKey
	extends DemultiplexKey
{
	PACKAGE final int offset;
	PACKAGE final byte[] value;

	public SimpleDemultiplexKey(int offset, byte[] value)
	{
		super(value.length);
		this.offset = offset;
		this.value = value;
	}

	public int matches(byte[] data, int packetOffset, int packetLength)
	{
		return compareSlice(data, packetOffset, packetLength,
		  		    this.offset, this.value);
	}

	public boolean equals(Object obj)
	{
		boolean retval = false;
		
		if( (obj instanceof SimpleDemultiplexKey) &&
		    (this.getClass().equals(obj.getClass())) )
		{
			if( obj == this )
				retval = true;
			else
			{
				SimpleDemultiplexKey sdk =
					(SimpleDemultiplexKey)obj;
				int myLength, sdkLength;

				myLength = this.value.length - this.offset;
				sdkLength = sdk.value.length - sdk.offset;
				if( myLength == sdkLength )
				{
					int myPos, sdkPos;
					
					myPos = this.offset;
					sdkPos = sdk.offset;
					retval = true;
					while( (myPos < myLength) &&
					       retval )
					{
						retval = this.value[myPos] ==
							sdk.value[sdkPos];
						myPos++;
						sdkPos++;
					}
				}
			}
		}
		return retval;
	}
}
